/**
 * File: Robojax_SevenSegmentClock.cpp
  Update by Ahmad Shamshiri (Robojax.com) on May 25, 2019
  
 * Description:
 * Robojax_SevenSegmentClock is an Arduino library for displaying analog values in LCD display, 
 *   which is previously initialized. This library uses LCD library (https://bitbucket.org/fmalpartida/new-liquidcrystal/wiki/Home) for displaying.
 *
 * Author: Ahmad Shamshiri
 * Contact: Robojax.com contact page
 * Copyright: 2019 Ahmad Shamshiri
 * Copying permission statement:
    This file is part of Robojax_SevenSegmentClock.

    Robojax_SevenSegmentClock is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "Arduino.h"
#include "Robojax_SevenSegmentClock.h"



static byte displayLEDs[10][7] = { 
        { 0,0,0,0,0,0,1 },  // = 0
        { 1,0,0,1,1,1,1 },  // = 1
        { 0,0,1,0,0,1,0 },  // = 2
        { 0,0,0,0,1,1,0 },  // = 3
        { 1,0,0,1,1,0,0 },  // = 4
        { 0,1,0,0,1,0,0 },  // = 5
        { 0,1,0,0,0,0,0 },  // = 6
        { 0,0,0,1,1,1,1 },  // = 7
        { 0,0,0,0,0,0,0 },  // = 8
        { 0,0,0,0,1,0,0 }   // = 9     
        }; 



// -- constructor
Robojax_SevenSegmentClock::Robojax_SevenSegmentClock(int segments[], int digits[], int colon, int type)
{
    for(int i=0; i<7; i++)
	{
		this->_pinSegments[i]=segments[i];
	}
    for(int j=0; j<4; j++)
	{
		this->_pinDigits[j] =digits[j];
	}

	this->_pinColon = colon;
	this->_type = type;
}

void Robojax_SevenSegmentClock::begin()
{

    for(int i=0; i<7; i++)
	{
		pinMode(this->_pinSegments[i], OUTPUT);
	}
    for(int j=0; j<4; j++)
	{
		pinMode(this->_pinDigits[j], OUTPUT);
	}
	pinMode(_pinColon, OUTPUT);
}

// -- write time like 02:35 to the seven segment
void Robojax_SevenSegmentClock::displayTime(int hour, int minute) {
	int digits[4];
  digits[0] = minute % 10;
  digits[1] =(minute / 10) % 10; 	
  digits[2] = hour % 10;  	
  digits[3] =(hour / 10) % 10;
 

	this->clearAllDigits();

		for(int j=0; j<4; j++)
		{
			this->digitOn(j);
			this->writeDigit(j,digits[j]);
			delay(5);
			this->clearAllDigits();
			//Serial.print(digits[j]);
		}
    this->colonBlink();// send signal to blink

}//displayTime end


// -- write single digit to the seven segment
/*
 * 
 * @brief writes number to a digit 
 * @param digit (integer), number (integer)
 * @return no return value
 * Written by Ahmad Shamshiri on July 28, 2019 for Robojax.com
 */    
void Robojax_SevenSegmentClock::writeDigit(int digit, int number) {
if(this->debug ==1){
  Serial.print("Digit:");
  Serial.print(digit);
  Serial.print(" as:");
  Serial.print(this->convertHighLow(1));  
  Serial.print(" number:");  
  Serial.print(number);
  Serial.print(" ");
}
   int digitValue;
  for (int i=0; i < 7; i++) 
  {
      digitalWrite(this->_pinDigits[digit],this->convertHighLow(1));
      digitalWrite(this->_pinSegments[i], this->convertHighLow(displayLEDs[number][i]));// write the HGIH or LOW (depending on the type) to output pin
      if(this->debug ==1){Serial.print(this->convertHighLow(displayLEDs[number][i]));}// print the result to the Serial monitor

  }// for i
  if(this->debug ==1){
  Serial.println();
  }

}//writeDigit end


/*
*makes the colon to blink every second
half of second ON and half of second OFF
*
*/
void Robojax_SevenSegmentClock::colonBlink()
{
  if( (millis() - this->_rememBlinkTime) >= this->_blinkTime)
  {

	digitalWrite(this->_pinColon, this->convertHighLow(this->_blinkStatus));	
    this->_blinkStatus = 1 - this->_blinkStatus;// convert 1 to 0 or 0 to 1
    this->_rememBlinkTime =millis();	
  }	


}

/*
* turns specific digit ON
*
*/
void Robojax_SevenSegmentClock::digitOn(int digit){
  digitalWrite(this->_pinDigits[digit],this->convertHighLow(1));
}//digitOn

/*
* clears specific digit from previous values
*
*/
void Robojax_SevenSegmentClock::digitOff(int digit){
  digitalWrite(this->_pinDigits[digit],this->convertHighLow(0));
}//digitOff

/*
* clears all  digit from previous values
*
*/
void Robojax_SevenSegmentClock::clearAllDigits(){
  digitalWrite(this->_pinDigits[0],this->convertHighLow(0));
  digitalWrite(this->_pinDigits[1],this->convertHighLow(0));
  digitalWrite(this->_pinDigits[2],this->convertHighLow(0));
  digitalWrite(this->_pinDigits[3],this->convertHighLow(0));
}//clearAllDigits(

/*
 * 
 * @brief converts HIGH and LOW based on the "type" variable value
 * if type is 1, converts the "V" to LOW, or returns it as it is if type if 0 (zero)
 * @param v (integer)
 * @return the convertedValue
 * Written by Ahmad Shamshiri (Robojax.com) on Friday Dec 08, 2018 for Robojax.com
 */  
int Robojax_SevenSegmentClock::convertHighLow(int v)
{
  int convertedValue;
    if( this->_type ==0)
    {
      convertedValue = 1 - v;// if common anode, change the HIGH to LOW and LOW to HIGH
    }else{
      convertedValue = v;// if common cathode, keep it the same
    } 

    return convertedValue;
}


